﻿using Asp.Versioning.ApiExplorer;
using Infrastructure.Seed;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Diagnostics;
using Serilog;

namespace Infrastructure.Extensions;

public static class ApplicationBuilderExtension
{
    public static IApplicationBuilder UseDefaultSwaggerUI(this IApplicationBuilder app)
    {
        ArgumentNullException.ThrowIfNull(app);
        var apiVersionDescriptionProvider =
            app.ApplicationServices.GetRequiredService<IApiVersionDescriptionProvider>();
        app.UseSwaggerUI(options =>
        {
            foreach (var description in apiVersionDescriptionProvider.ApiVersionDescriptions)
            {
                options.SwaggerEndpoint($"/swagger/{description.GroupName}/swagger.json",
                    description.GroupName.ToUpperInvariant());
            }
        });
        return app;
    }

    public static IApplicationBuilder UseDefaultExceptionHandler(this IApplicationBuilder app)
    {
        ArgumentNullException.ThrowIfNull(app);
        app.UseExceptionHandler(options => 
        {
            options.Run(async context=>
            {
                context.Response.ContentType = "application/json";
                context.Response.StatusCode = StatusCodes.Status200OK;
                var exceptionHandlerPathFeature =
                    context.Features.Get<IExceptionHandlerPathFeature>();
                var message = new MessageData(false, "unknown error", 500);
                Log.Logger.Error(exceptionHandlerPathFeature?.Error.Message!);
                await context.Response.WriteAsync(message.Serialize());
            });   
        });

        return app;
    }
}